@@ -13,6 +13,9 @@ map_arrow_to_numpy[pa.bool_()] = np.dtype("?")
 for unit in 's ms us ns'.split():
     map_arrow_to_numpy[pa.timestamp(unit)] = np.dtype(f"datetime64[{unit}]")
 
+for unit in 's ms us ns'.split():
+    map_arrow_to_numpy[pa.duration(unit)] = np.dtype(f"timedelta64[{unit}]")
+
 
 def full(n, value, dtype):
     from .datatype import DataType
@@ -35,6 +38,9 @@ def is_array(ar):
     return is_arrow_array(ar) or is_numpy_array(ar)
 
 
+def is_scalar(x):
+    return not is_array(x) or (is_numpy_array(x) and x.ndim == 0)
+
 def filter(ar, boolean_mask):
     if isinstance(ar, supported_arrow_array_types):
         return ar.filter(pa.array(boolean_mask))
@@ -42,6 +48,10 @@ def filter(ar, boolean_mask):
         return ar[boolean_mask]
 
 
+def take(ar, indices):
+    return ar.take(indices)
+
+
 def slice(ar, offset, length=None):
     if offset == 0 and len(ar) == length:
         return ar
@@ -54,6 +64,28 @@ def slice(ar, offset, length=None):
             return ar[offset:]
 
 
+def shape(ar):
+    if is_arrow_array(ar):
+        return (len(ar),)
+    else:
+        return ar.shape
+
+def ndim(ar):
+    if is_arrow_array(ar):
+        return 1
+    else:
+        return ar.ndim
+
+
+def getitem(ar, item):
+    if is_arrow_array(ar):
+        assert len(item) == 1, "For arrow we only support 1 d items"
+        assert item[0].step is None, "Step not supported for arrow"
+        start, end = item[0].start, item[0].stop
+        return ar[start:end]
+    else:
+        return ar[item]
+
 def concat(arrays):
     if len(arrays) == 1:
         return arrays[0]
@@ -131,7 +163,8 @@ def to_numpy(x, strict=False):
     elif isinstance(x, np.ndarray):
         return x
     elif isinstance(x, supported_arrow_array_types):
-        if not strict and is_string(x):
+        dtype = vaex.dtype_of(x)
+        if not strict and not (dtype.is_primitive or dtype.is_temporal):
             return x
         x = vaex.arrow.convert.column_from_arrow_array(x)
         return to_numpy(x, strict=strict)
@@ -141,6 +174,9 @@ def to_numpy(x, strict=False):
 
 
 def to_arrow(x, convert_to_native=True):
+    if isinstance(x, (vaex.strings.StringList32, vaex.strings.StringList64)):
+        col = vaex.column.ColumnStringArrow.from_string_sequence(x)
+        return pa.array(col)
     if isinstance(x, supported_arrow_array_types):
         return x
     if convert_to_native and isinstance(x, np.ndarray):
