@@ -7,6 +7,7 @@ import six
 import numpy as np
 import pyarrow as pa
 from vaex.dataframe import DataFrame
+from vaex.docstrings import docsubst
 
 import vaex.array_types
 from vaex.delayed import delayed_args, delayed_dict, delayed_list
@@ -28,7 +29,7 @@ _EXPERIMENTAL_BINNER_HASH = False
 
 
 # pure Python to avoid int overflow
-product = lambda l: reduce(operator.mul, l)
+product = lambda l: reduce(operator.mul, l, 1)
 
 
 class BinnerBase:
@@ -76,12 +77,12 @@ class BinnerTime(BinnerBase):
     >>> y = np.arange(len(t))
     >>> df = vaex.from_arrays(t=t, y=y)
     >>> df.groupby(vaex.BinnerTime.per_week(df.t)).agg({'y' : 'sum'})
-    #  t                      y
-    0  2015-01-01 00:00:00   21
-    1  2015-01-08 00:00:00   70
-    2  2015-01-15 00:00:00  119
-    3  2015-01-22 00:00:00  168
-    4  2015-01-29 00:00:00   87
+      #  t             y
+      0  2015-01-01   21
+      1  2015-01-08   70
+      2  2015-01-15  119
+      3  2015-01-22  168
+      4  2015-01-29   87
 
     """
 
@@ -559,9 +560,9 @@ def _combine(df, groupers, sort, row_limit=None, progress=None):
     for i in range(0, len(combine_expressions)):
         combine_expression = combine_expressions[i]
         dtype = vaex.utils.required_dtype_for_max(cumulative_counts[i])
-        combine_expression = combine_expression.astype(str(dtype))
         if isinstance(combine_now[i], (GrouperCategory, BinnerInteger)) and combine_now[i].min_value != 0:
             combine_expression -= combine_now[i].min_value
+        combine_expression = combine_expression.astype(str(dtype))
         if cumulative_counts[i + 1] != 1:
             combine_expression = combine_expression * cumulative_counts[i + 1]
         combine_expressions[i] = combine_expression
@@ -606,7 +607,9 @@ class GroupByBase(object):
         self.progressbar = vaex.utils.progressbars(progress)
         self.progressbar_groupers = self.progressbar.add("groupers")
 
-        if not isinstance(by, collections_abc.Iterable)\
+        if by is None:
+            by = []
+        elif not isinstance(by, collections_abc.Iterable)\
             or isinstance(by, six.string_types):
             by = [by]
 
@@ -754,20 +757,19 @@ class GroupByBase(object):
         >>> g1 = df.groupby(by='pclass')
         >>> df_group1 = g1.get_group(1)
         >>> df_group1.head(3)
-          #  name                              pclass  sex         age     fare
-          0  Allen, Miss. Elisabeth Walton          1  female  29       211.338
-          1  Allison, Master. Hudson Trevor         1  male     0.9167  151.55
-          2  Allison, Miss. Helen Loraine           1  female   2       151.55
-
+          #    pclass  survived    name                            sex         age    sibsp    parch    ticket     fare  cabin    embarked    boat      body  home_dest
+          0         1  True        Allen, Miss. Elisabeth Walton   female  29             0        0     24160  211.338  B5       S           2          nan  St Louis, MO
+          1         1  True        Allison, Master. Hudson Trevor  male     0.9167        1        2    113781  151.55   C22 C26  S           11         nan  Montreal, PQ / Chesterville, ON
+          2         1  False       Allison, Miss. Helen Loraine    female   2             1        2    113781  151.55   C22 C26  S           --         nan  Montreal, PQ / Chesterville, ON
 
         >>> df = vaex.datasets.titanic()
         >>> g2 = df.groupby(by=['pclass', 'sex'])
         >>> df_group2 = g2.get_group([1, 'female'])
         >>> df_group2.head(3)
-          #  name                                               pclass  sex       age     fare
-          0  Allen, Miss. Elisabeth Walton                           1  female     29  211.338
-          1  Allison, Miss. Helen Loraine                            1  female      2  151.55
-          2  Allison, Mrs. Hudson J C (Bessie Waldo Daniels)         1  female     25  151.55
+          #    pclass  survived    name                                             sex       age    sibsp    parch    ticket     fare  cabin    embarked    boat      body  home_dest
+          0         1  True        Allen, Miss. Elisabeth Walton                    female     29        0        0     24160  211.338  B5       S           2          nan  St Louis, MO
+          1         1  False       Allison, Miss. Helen Loraine                     female      2        1        2    113781  151.55   C22 C26  S           --         nan  Montreal, PQ / Chesterville, ON
+          2         1  False       Allison, Mrs. Hudson J C (Bessie Waldo Daniels)  female     25        1        2    113781  151.55   C22 C26  S           --         nan  Montreal, PQ / Chesterville, ON
 
         :param group: A value or a list of values of the expressions used to create the groupby object.
             If `assume_sparse=True` when greating the groupby object, this param takes an int corresponding to the particular groupby combination.
@@ -885,11 +887,22 @@ class GroupBy(GroupByBase):
         def process(args):
             counts, arrays = args
             logger.info(f"aggregated on grid, constructing dataframe...")
+            if counts is not None and vaex.array_types.is_scalar(counts):
+                counts = np.array([counts])
+
+
+            for name, array in list(arrays.items()):
+                if array is not None and vaex.array_types.is_scalar(array):
+                    array = np.array([array])
+                    arrays[name] = array
+
             if counts is not None:
                 for name, array in arrays.items():
 
-                    if vaex.array_types.shape(array) != vaex.array_types.shape(counts):
-                        raise RuntimeError(f'{array} {name} has shape {array.shape} while we expected {counts.shape}')
+                    shape1 = vaex.array_types.shape(array)
+                    shape2 = vaex.array_types.shape(counts)
+                    if shape1 != shape2:
+                        raise RuntimeError(f'{array} {name} has shape {shape1} while we expected {shape2}')
 
             arrays = {key: self._extract_center(value) for key, value in arrays.items()}
             if not self.dense:
@@ -915,7 +928,7 @@ class GroupBy(GroupByBase):
                 values = self.by[0].bin_values
                 columns = {field.name: ar for field, ar in zip(values.type, values.flatten())}
                 for key, value in arrays.items():
-                    assert value.ndim == 1
+                    assert vaex.array_types.ndim(value) == 1
                     columns[key] = value
             else:
                 columns = {}
@@ -960,6 +973,37 @@ class GroupBy(GroupByBase):
             self.df.execute()
             return result.get()
 
+    @docsubst
+    def describe(self, expression=None):
+        '''Return a dataframe with summary statistics per group for each of the expressions provided.
+
+        Example:
+
+        >>> import vaex
+        >>> df = vaex.datasets.titanic()
+        >>> df.groupby('pclass').describe('age')
+          #    pclass    age_count    age_count_na    age_mean    age_std    age_min    age_max
+          0         1          284              39     39.1599    14.5224     0.9167         80
+          1         2          261              16     29.5067    13.6125     0.6667         70
+          2         3          501             208     24.8164    11.9463     0.1667         74
+
+        Short for:
+        >>> df.groupby('pclass').agg({{'age': vaex.agg.describe('age')}}).struct.flatten()
+          #    pclass    age_count    age_count_na    age_mean    age_std    age_min    age_max
+          0         1          284              39     39.1599    14.5224     0.9167         80
+          1         2          261              16     29.5067    13.6125     0.6667         70
+          2         3          501             208     24.8164    11.9463     0.1667         74
+
+        :param expression: {expression}
+        :rtype: DataFrame
+        '''
+        if expression is None:
+            expression = self.df.get_column_names()
+        columns = vaex.utils._ensure_strings_from_expressions(expression)
+        columns = vaex.utils._ensure_list(columns)
+        aggs = {col: vaex.agg.describe(col) for col in columns}
+        return self.agg(aggs).struct.flatten()
+
 
 @vaex.dataset.register
 class DatasetGroupby(vaex.dataset.DatasetDecorator):
