@@ -123,6 +123,31 @@ class Data(BaseSettings):
     class Config(ConfigDefault):
         env_prefix = 'vaex_data_'
 
+class Progress(BaseSettings):
+    """Data configuration"""
+    type: str = Field('simple', title="Default progressbar to show: 'simple', 'rich' or 'widget'")
+    force: str = Field(None, title="Force showing a progress bar of this type, even when no progress bar was requested from user code", env="VAEX_PROGRESS")
+
+    class Config(ConfigDefault):
+        env_prefix = 'vaex_progress_'
+
+
+class Logging(BaseSettings):
+    """Configure logging for Vaex. By default Vaex sets up logging, which is useful when running a script. When Vaex is used in applications or services that already configure logging, set the environomental variables VAEX_LOGGING_SETUP to false.
+
+See the [API docs](api.html#module-vaex.logging) for more details.
+
+Note that settings `vaex.settings.main.logging.info` etc at runtime, has no direct effect, since logging is already configured. When needed, call `vaex.logging.reset()` and `vaex.logging.setup()` to reconfigure logging.
+    """
+    setup : bool = Field(True, title='Setup logging for Vaex at import time.')
+    rich : bool = Field(True, title='Use rich logger (colored fancy output).')
+    debug : str = Field('', title="Comma seperated list of loggers to set to the debug level (e.g. 'vaex.settings,vaex.cache'), or a '1' to set the root logger ('vaex')")
+    info : str = Field('', title="Comma seperated list of loggers to set to the info level (e.g. 'vaex.settings,vaex.cache'), or a '1' to set the root logger ('vaex')")
+    warning : str = Field('vaex', title="Comma seperated list of loggers to set to the warning level (e.g. 'vaex.settings,vaex.cache'), or a '1' to set the root logger ('vaex')")
+    error : str = Field('', title="Comma seperated list of loggers to set to the error level (e.g. 'vaex.settings,vaex.cache'), or a '1' to set the root logger ('vaex')")
+    class Config(ConfigDefault):
+        env_prefix = 'vaex_logging_'
+
 
 class Settings(BaseSettings):
     """General settings for vaex"""
@@ -134,6 +159,8 @@ class Settings(BaseSettings):
     process_count: Optional[int] = Field(title="Number of processes to use for multiprocessing (e.g. apply), defaults to thread_count setting", gt=0)
     thread_count: Optional[int] = Field(env='VAEX_NUM_THREADS', title="Number of threads to use for computations, defaults to multiprocessing.cpu_count()", gt=0)
     thread_count_io: Optional[int] = Field(env='VAEX_NUM_THREADS_IO', title="Number of threads to use for IO, defaults to thread_count_io + 1", gt=0)
+    path_lock: str = Field(os.path.join(_default_home, "lock"), env="VAEX_LOCK", title="Directory to store lock files for vaex, which defaults to `${VAEX_HOME}/lock/`, "\
+        " Due to possible race conditions lock files cannot be removed while processes using Vaex are running (on Unix systems).")
 
 
     # avoid name collisions of VAEX_CACHE with configurting the whole object via json in env var
@@ -144,6 +171,8 @@ class Settings(BaseSettings):
     fs: FileSystem = Field(FileSystem(), env='_VAEX_FS')
     memory_tracker = Field(MemoryTracker(), env='_VAEX_MEMORY_TRACKER')
     task_tracker = Field(TaskTracker(), env='_VAEX_TASK_TRACKER')
+    logging = Field(Logging(), env="_VAEX_LOGGING")
+    progress = Field(Progress(), env="_VAEX_PROGRESS")
 
     if has_server:
         server: vaex.server.settings.Settings = vaex.server.settings.Settings()
@@ -254,9 +283,11 @@ def _to_md(cls, f=sys.stdout):
             Cache: 'cache',
             FileSystem: 'fs',
             Data: 'data',
+            Logging: 'main.logging',
+            Progress: 'main.progress',
         }[cls]
-        pyvar = f'vaex.setting.{flat}.{pyname}'
-        printf(f'Python setting `{pyvar}`')
+        pyvar = f'vaex.settings.{flat}.{pyname}'
+        printf(f'Python settings `{pyvar}`')
         if default not in [None, ""]:
             printf()
             printf(f'Example use: `{pyvar} = {default!r}`')
