@@ -1,7 +1,15 @@
+import difflib
 import logging
+import os
+import operator
+from functools import reduce
+import threading
+import pkg_resources
+
 import numpy as np
 
 import vaex.promise
+import vaex.settings
 import vaex.encoding
 from .utils import _expand_shape
 from .datatype import DataType
@@ -10,31 +18,94 @@ from .datatype import DataType
 logger = logging.getLogger('vaex.tasks')
 register = vaex.encoding.make_class_registery('task')
 
+_lock = threading.Lock()
+_task_checker_types = {}
+
+
+class Checker:
+    track_live = False
+
+    def add_task(self, task):
+        pass
+
+    def add_task_part(self, task):
+        pass
+
+
+def create_checkers():
+    if not _task_checker_types:
+        with _lock:
+            if not _task_checker_types:
+                for entry in pkg_resources.iter_entry_points(group="vaex.task.checker"):
+                    _task_checker_types[entry.name] = entry.load()
+    names = list(_task_checker_types.keys())
+    task_checkers_type = vaex.settings.main.task_tracker.type
+    types = [k for k in task_checkers_type.split(",") if k]
+    checkers = []
+    for type in types:
+        cls = _task_checker_types.get(type)
+        if cls is not None:
+            checkers.append(cls())
+        else:
+            msg = f"Task checker {type} does not exist."
+            matches = difflib.get_close_matches(type, names)
+            if matches:
+                msg += " Did you mean: " + " or ".join(map(repr, matches))
+            else:
+                msg += " Options are: " + repr(names)
+            raise NameError(msg)
+    return checkers
+
+
+class TaskCheckError(RuntimeError):
+    pass
+
 
 class Task(vaex.promise.Promise):
     """
     :type: signal_progress: Signal
     """
     _fingerprint = None
+    requires_fingerprint = False
     cacheable = True
+    see_all = False
+    _toreject: Exception = None
 
     def __init__(self, df=None, expressions=[], pre_filter=False, name="task"):
         vaex.promise.Promise.__init__(self)
         self.df = df
         self.expressions = expressions
         self.expressions_all = list(expressions)
+        self.signal_start = vaex.events.Signal("start of task")
         self.signal_progress = vaex.events.Signal("progress (float)")
         self.progress_fraction = 0
         self.signal_progress.connect(self._set_progress)
         self.cancelled = False
+        self.stopped = False  # a task can stop early, but without error
         self.name = name
         self.pre_filter = pre_filter
         self.result = None
 
+    def dependencies(self):
+        variables = set()
+        for expression in self.expressions_all:
+            variables |= self.df._expr(expression).dependencies()
+        for expression in self.selections:
+            variables |= self.df._selection_expression(expression).dependencies()
+        return variables
+
     def fingerprint(self):
         if self._fingerprint is None:
-            self._fingerprint = vaex.encoding.fingerprint('task', self)
-        return f'task-{self.name}-{self._fingerprint}'
+            dependencies = self.dependencies()
+            df_fp = self.df.fingerprint(dependencies=dependencies)
+            task_fp = vaex.encoding.fingerprint('task', self)
+            self._fingerprint = f'task-{self.name}-{task_fp}-{df_fp}'
+        return self._fingerprint
+
+    def progress(self, fraction):
+        if not self.cancelled:
+            self.cancelled = not all(self.signal_progress.emit(fraction))
+        return not self.cancelled
 
     def _set_progress(self, fraction):
         self.progress_fraction = fraction
@@ -54,7 +125,7 @@ class Task(vaex.promise.Promise):
 
     def create_next(self):
         ret = Task(self.df, [])
-        self.signal_progress.connect(ret.signal_progress.emit)
+        self.signal_progress.connect(lambda f: all(ret.signal_progress.emit(f)))
         return ret
 
     # def __repr__(self):
@@ -68,6 +139,9 @@ class Task(vaex.promise.Promise):
 class TaskSum(Task):
     snake_name = "sum-test"
 
+    def get_bin_count(self):
+        return 1
+
     # def __init__(self, df, expression):
     #     super().__init__(df, expression)
     #     self.expression = expression
@@ -86,6 +160,10 @@ class TaskFilterFill(Task):
     snake_name = "filter_fill"
     def __init__(self, df):
         super().__init__(df=df, pre_filter=True, name=self.snake_name)
+        self.selections = []
+
+    def get_bin_count(self):
+        return 0
 
     def encode(self, encoding):
         return {}
@@ -95,25 +173,74 @@ class TaskFilterFill(Task):
         return cls(df)
 
 @register
-class TaskSetCreate(Task):
-    snake_name = "set_create"
-    def __init__(self, df, expression, flatten, unique_limit=None, selection=None):
+class TaskHashmapUniqueCreate(Task):
+    see_all = True
+    requires_fingerprint = True
+    snake_name = "hash_map_unique_create"
+    def __init__(self, df, expression, flatten, limit=None, limit_raise=True, selection=None, return_inverse=False):
         super().__init__(df=df, expressions=[expression], pre_filter=df.filtered, name=self.snake_name)
         self.flatten = flatten
         self.dtype = self.df.data_type(expression)
         self.dtype_item = self.df.data_type(expression, axis=-1 if flatten else 0)
-        self.unique_limit = unique_limit
+        self.limit = limit
+        self.limit_raise = limit_raise
         self.selection = selection
+        self.selections = [self.selection]
+        self.return_inverse = return_inverse
+
+    def get_bin_count(self):
+        return 0
+
+    def __repr__(self):
+        return f"task-{self.snake_name}: expression={self.expressions[0]!r}"
 
     def encode(self, encoding):
         return {'expression': self.expressions[0], 'dtype': encoding.encode('dtype', self.dtype),
-                'dtype_item': encoding.encode('dtype', self.dtype_item), 'flatten': self.flatten, 'unique_limit': self.unique_limit,
-                'selection': self.selection}
+                'dtype_item': encoding.encode('dtype', self.dtype_item), 'flatten': self.flatten,
+                'limit': self.limit, 'limit_raise': self.limit_raise,
+                'selection': self.selection, 'return_inverse': self.return_inverse}
 
     @classmethod
     def decode(cls, encoding, spec, df):
         return cls(df, expression=spec['expression'])
 
+
+class TaskValueCounts(Task):
+    snake_name = "value_counts"
+
+    def __init__(self, df, expression, dropnan=False, dropmissing=False, ascending=False, axis=None):
+        super().__init__(df=df, expressions=[expression], pre_filter=df.filtered, name=self.snake_name)
+        self.flatten = axis is None
+        self.dtype = self.df.data_type(expression)
+        self.dtype_item = self.df.data_type(expression, axis=-1 if axis is None else 0)
+        self.selection = None
+        self.selections = [self.selection]
+        self.dropnan = dropnan
+        self.dropmissing = dropmissing
+        self.ascending = ascending
+
+    def get_bin_count(self):
+        return 0
+
+    def __repr__(self):
+        return f"task-{self.snake_name}: expression={self.expressions[0]!r}"
+
+    def encode(self, encoding):
+        return {
+            "expression": self.expressions[0],
+            "dtype": encoding.encode("dtype", self.dtype),
+            "dtype_item": encoding.encode("dtype", self.dtype_item),
+            "flatten": self.flatten,
+            "dropnan": self.dropnan,
+            "dropmissing": self.dropmissing,
+            "ascending": self.ascending,
+        }
+
+    @classmethod
+    def decode(cls, encoding, spec, df):
+        return cls(df, expression=spec["expression"])
+
+
 @register
 class TaskMapReduce(Task):
     snake_name = "map_reduce"
@@ -132,6 +259,10 @@ class TaskMapReduce(Task):
         if self.pre_filter and self.ignore_filter:
             raise ValueError("Cannot pre filter and also ignore the filter")
         self.selection = selection
+        self.selections = [self.selection]
+
+    def get_bin_count(self):
+        return 0
 
     def encode(self, encoding):
         return {'expressions': self.expressions, 'map': self._map, 'reduce': self._reduce,
@@ -139,6 +270,9 @@ class TaskMapReduce(Task):
                 'skip_masked': self.skip_masked, 'ignore_filter': self.ignore_filter, 'selection': self.selection, 'pre_filter': self.pre_filter,
                 }
 
+    def __repr__(self) -> str:
+        return f'TaskMapReduce(map={self._map}, reduce={self._reduce}'
+
 
 class StatOp(object):
     def __init__(self, code, fields, reduce_function=np.nansum, dtype=None):
@@ -267,6 +401,9 @@ class TaskStatistic(Task):
                 'dtype': encoding.encode('dtype', DataType(self.dtype)), 'minima': self.minima, 'maxima': self.maxima, 'edges': self.edges,
                 'selection_waslist': self.selection_waslist}
 
+    def get_bin_count(self):
+        return reduce(operator.mul, self.shape, 1)
+
     @classmethod
     def decode(cls, encoding, spec, df):
         spec = spec.copy()
@@ -326,6 +463,7 @@ class TaskStatistic(Task):
 class TaskAggregations(Task):
     """Multiple aggregations on a single grid."""
     snake_name = "aggregations"
+    see_all = True
 
     def __init__(self, df, binners):
         expressions = [binner.expression for binner in binners]
@@ -335,6 +473,10 @@ class TaskAggregations(Task):
         self.dtypes = {}
         Task.__init__(self, df, expressions, name="statisticNd", pre_filter=df.filtered)
         self.original_tasks = []
+        self.selections = []
+
+    def get_bin_count(self):
+        return reduce(lambda prev, binner: binner.count * prev, self.binners, 1)
 
     def __repr__(self):
         encoding = vaex.encoding.Encoding()
@@ -373,6 +515,13 @@ class TaskAggregations(Task):
         self.then(assign_subtask, chain_reject)
 
         self.aggregation_descriptions.append((aggregator_descriptor))
+        if aggregator_descriptor.selection is not None:
+            if isinstance(aggregator_descriptor.selection, (list, tuple)):
+                self.selections.extend([str(selection) if selection is not None else None for selection in aggregator_descriptor.selection])
+            else:
+                self.selections.append(str(aggregator_descriptor.selection))
+        else:
+            self.selections.append(None)
         # THIS SHOULD BE IN THE SAME ORDER AS THE ABOVE TASKPART
         # it is up the the executor to remove duplicate expressions
         self.expressions_all.extend(aggregator_descriptor.expressions)
@@ -399,12 +548,10 @@ class TaskAggregation(Task):
         Task.__init__(self, df, expressions, name=self.snake_name, pre_filter=df.filtered)
         self.dtypes = {expr: self.df.data_type(expr).index_type for expr in self.expressions_all}
         self.expressions_all.extend(aggregation_description.expressions)
+        self.selections = [str(aggregation_description.selection) if aggregation_description.selection is not None else None]
 
     def __repr__(self):
-        encoding = vaex.encoding.Encoding()
-        state = self.encode(encoding)
-        import yaml
-        return yaml.dump(state, sort_keys=False, indent=4)
+        return f"task-{self.snake_name} agg={self.aggregation_description!r} selection={self.selections[0]!r} binners=[{self.binners!r}]"
 
     def encode(self, encoding):
         # TODO: get rid of dtypes
